//-----------------------------------------------------------------------------
// Torque Game Engine
//
// Copyright (c) 2001 GarageGames.Com
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Penalty and bonus times.
$Game::TimeTravelBonus = 5000;

// Item respawn values, only powerups currently respawn
$Item::RespawnTime = 7 * 1000;
$Item::PopTime = 10 * 1000;

// Game duration in secs, no limit if the duration is set to 0
$Game::Duration = 0;

// Pause while looking over the end game screen (in secs)
$Game::EndGamePause = 5;

//-----------------------------------------------------------------------------
// Variables extracted from the mission
$Game::GemCount = 0;
$Game::StartPad = 0;
$Game::EndPad = 0;
$Game::Checkpointed = 0;

// Spy47 : Change this if you want !!
$powerupDelay = 600;


//-----------------------------------------------------------------------------
//  Functions that implement game-play
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

function onServerCreated()
{
   // Server::GameType is sent to the master server.
   // This variable should uniquely identify your game and/or mod.
   $Server::GameType = "Marble Game";

   // Server::MissionType sent to the master server.  Clients can
   // filter servers based on mission type.
   $Server::MissionType = "Deathmatch";

   // GameStartTime is the sim time the game started. Used to calculated
   // game elapsed time.
   $Game::StartTime = 0;

   // Load up all datablocks, objects etc.  This function is called when
   // a server is constructed.
   exec("./audioProfiles.cs");
   exec("./camera.cs");
   exec("./markers.cs");
   exec("./triggers.cs");
   exec("./inventory.cs");
   exec("./shapeBase.cs");
   exec("./staticShape.cs");
   exec("./item.cs");

   // Basic items
   exec("./marble.cs");
   exec("./gems.cs");
   exec("./powerUps.cs");
   exec("./buttons.cs");
   exec("./hazards.cs");
   exec("./pads.cs");
   exec("./bumpers.cs");
   exec("./signs.cs");
   exec("./fireworks.cs");
   exec("./checkpoint.cs");
   exec("./teleporter.cs");

   // Marble Blast Gold stuff
   exec("./MarbleBlastGold.cs");

   // Platforms and interior doors
   exec("./pathedInteriors.cs");

   // Keep track of when the game started
   $Game::StartTime = $Sim::Time;
}

function onServerDestroyed()
{
   // Perform any game cleanup without actually ending the game
   destroyGame();
}


//-----------------------------------------------------------------------------

function onMissionLoaded()
{
   // Called by loadMission() once the mission is finished loading.
   // Nothing special for now, just start up the game play.

   $Game::GemCount = countGems(MissionGroup);

   setGravityDir("1 0 0 0 -1 0 0 0 -1",true);
   $Game::GravityDir = "1 0 0 0 -1 0 0 0 -1";

   // Start the game here if multiplayer...
   if ($Server::ServerType $= "MultiPlayer")
      startGame();
}

function onMissionEnded()
{
   // Called by endMission(), right before the mission is destroyed
   // This part of a normal mission cycling or end.
   //$enableEditor = 0;
   endGame();
}

function onMissionReset()
{
   setGravityDir("1 0 0 0 -1 0 0 0 -1",true);
   $Game::GravityDir = "1 0 0 0 -1 0 0 0 -1";
   endFireWorks();
   
   // Reset the players and inform them we're starting
   for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
      %cl = ClientGroup.getObject( %clientIndex );
      commandToClient(%cl, 'GameStart');
      %cl.resetStats();
   }

   // Start the game duration timer
   if ($Game::Duration)
      $Game::CycleSchedule = schedule($Game::Duration * 1000, 0, "onGameDurationEnd" );
   $Game::Running = true;

   ServerGroup.onMissionReset();

   // Set the initial state
   setGameState("Start");

   $Game::Checkpointed = 0; // Spy47 : Reset checkpoint counter.
   $Game::CurCheckpoint = 0;
   $Game::CurCheckpointNum = 0;
}

function SimGroup::onMissionReset(%this)
{
   for(%i = 0; %i < %this.getCount(); %i++)
      %this.getObject(%i).onMissionReset();
}

function SimObject::onMissionReset(%this)
{
}

function GameBase::onMissionReset(%this)
{
   %this.getDataBlock().onMissionReset(%this);
}

//-----------------------------------------------------------------------------

function startGame()
{
   if ($Game::Running) {
      error("startGame: End the game first!");
      return;
   }
   $Game::Running = true;
   $Game::Qualified = false;
   onMissionReset();
}

function endGame()
{
   if (!$Game::Running) {
      error("endGame: No game running!");
      return;
   }
   destroyGame();

   // Inform the clients the game is over
   for (%index = 0; %index < ClientGroup.getCount(); %index++)  {
      %client = ClientGroup.getObject(%index);
      commandToClient(%client, 'GameEnd');
   }

   // Single player... grab the playgui as the elapsed time and
   // roll in clients penalty and bonus
   PlayGUI.stopTimer();

   $Game::ScoreTime = PlayGui.elapsedTime;
   $Game::ElapsedTime = PlayGui.elapsedTime - %client.penaltyTime + PlayGui.totalBonus;
   $Game::PenaltyTime = %client.penaltyTime;
   $Game::BonusTime = PlayGui.totalBonus;

   // Not all missions have time qualifiers
   $Game::Qualified = MissionInfo.time? $Game::ScoreTime < MissionInfo.time: true;

   // Bump up the max level
   if (!$DemoBuild && !$playingDemo && $Game::Qualified && (MissionInfo.level + 1) > $Pref::QualifiedLevel[MissionInfo.type])
         $Pref::QualifiedLevel[MissionInfo.type] = MissionInfo.level + 1;
}

function pauseGame()
{
   if ($Server::ServerType $= "SinglePlayer")
      $gamePaused = true;
}

function resumeGame()
{
   $gamePaused = false;
}

function destroyGame()
{
   // Cancel any client timers
   for (%index = 0; %index < ClientGroup.getCount(); %index++)
      cancel(ClientGroup.getObject(%index).respawnSchedule);

   // Perform cleanup to reset the game.
   cancel($Game::CycleSchedule);
   cancel($Game::StateSchedule);

   $Game::Running = false;
}


//-----------------------------------------------------------------------------

function setGameState(%state)
{
   cancel($Game::StateSchedule);
   $Game::State = %state;
   eval("State::"@%state@"();");
   echo("State "@%state@"");
}

function State::start()
{
   PlayGui.resetTimer();
   PlayGui.setMessage("");
   PlayGui.setGemCount(0);
   PlayGui.setMaxGems($Game::GemCount);
   $Game::StateSchedule = schedule( 500, 0, "setGameState", "Ready");
   if(MissionInfo.startHelpText !$= "")
   {
      addHelpLine(MissionInfo.startHelpText, false);
   }
}

function State::ready()
{
   serverPlay2d(ReadyVoiceSfx);
   PlayGui.setMessage("ready");
   $Game::StateSchedule = schedule( 1500, 0, "setGameState", "set");
}

function State::set()
{
   serverPlay2d(SetVoiceSfx);
   PlayGui.setMessage("set");
   $Game::StateSchedule = schedule( 1500, 0, "setGameState", "Go");
}

function State::go()
{
   serverPlay2d(GetRollingVoiceSfx);
   PlayGui.setMessage("go");
   PlayGui.startTimer();
   $Game::StateSchedule = schedule( 2000, 0, "setGameState", "Play");

   // Target the players to the end pad and let them lose
   for( %index = 0; %index < ClientGroup.getCount(); %index++ ) {
      %player = ClientGroup.getObject(%index).player;
      %player.setPad($Game::EndPad);
      %player.setMode(Normal);
   }
}

function State::play()
{
   // Normaly play mode
   PlayGui.setMessage("");
}

function State::end()
{
   // Do score calculations, messages to winner, losers, etc.
   PlayGUI.stopTimer();
   serverplay2d(WonRaceSfx);
   startFireWorks(EndPoint);
   $Game::StateSchedule = schedule( 2000, 0, "endGame");
}


//-----------------------------------------------------------------------------

function onGameDurationEnd()
{
   // This "redirect" is here so that we can abort the game cycle if
   // the $Game::Duration variable has been cleared, without having
   // to have a function to cancel the schedule.
   if ($Game::Duration && !isObject(EditorGui))
      cycleGame();
}

function cycleGame()
{
   // This is setup as a schedule so that this function can be called
   // directly from object callbacks.  Object callbacks have to be
   // carefull about invoking server functions that could cause
   // their object to be deleted.
   if (!$Game::Cycling) {
      $Game::Cycling = true;
      $Game::CycleSchedule = schedule(0, 0, "onCycleExec");
   }
}

function onCycleExec()
{
   // End the current game and start another one, we'll pause for a little
   // so the end game victory screen can be examined by the clients.
   endGame();
   $Game::CycleSchedule = schedule($Game::EndGamePause * 1000, 0, "onCyclePauseEnd");
}

function onCyclePauseEnd()
{
   $Game::Cycling = false;
   loadNextMission();
}

function loadNextMission()
{
   %nextMission = "";

   // Cycle to the next level, or back to the start if there aren't
   // any more levels.
   for (%file = findFirstFile($Server::MissionFileSpec);
         %file !$= ""; %file = findNextFile($Server::MissionFileSpec))
      if (strStr(%file, "CVS/") == -1 && strStr(%file, "common/") == -1)
      {
         %mission = getMissionObject(%file);
         if (%mission.type $= MissionInfo.type) {
            if (%mission.level == 1)
               %nextMission = %file;
            if ((%mission.level + 0) == MissionInfo.level + 1) {
               //echo("Found one!");
               %nextMission = %file;
               break;
            }
         }
      }
   loadMission(%nextMission);
}



//-----------------------------------------------------------------------------
// GameConnection Methods
// These methods are extensions to the GameConnection class. Extending
// GameConnection make is easier to deal with some of this functionality,
// but these could also be implemented as stand-alone functions.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

function GameConnection::incPenaltyTime(%this,%dt)
{
   PlayGui.adjustTimer(%dt);
   %this.penaltyTime += %dt;
}

function GameConnection::incBonusTime(%this,%dt)
{
   PlayGui.addBonusTime(%dt);
   %this.bonusTime += %dt;
}


//-----------------------------------------------------------------------------
function GameConnection::getMarbleChoice(%this, %var)
{
      switch( %var )
   	  {
      case 1:
         return MarbleOne;
      case 2:
         return MarbleTwo;
      case 3:
         return MarbleThree;
      case 4:
         return MarbleFour;
      case 5:
         return MarbleFive;
      case 6:
         return MarbleSix;
      case 7:
         return MarbleSeven;
      case 8:
         return MarbleEight;
      case 9:
         return MarbleNine;
      case 10:
         return MarbleTen;
      case 11:
         return MarbleEleven;
      case 12:
         return MarbleTwelve;
      case 13:
         return MarbleThirteen;
      case 14:
         return MarbleFourteen;
      case 15:
         return MarbleFifteen;
      case 16:
         return MarbleSixteen;
      case 17:
         return MarbleSeventeen;
      case 18:
         return MarbleEighteen;
      case 19:
         return MarbleNineteen;
      case 20:
         return MarbleTwenty;
      case 21:
         return MarbleTwentyOne;
      case 22:
         return MarbleTwentyTwo;
      case 23:
         return MarbleTwentyThree;
      case 24:
         return MarbleTwentyFour;
      case 25:
         return MarbleTwentyFive;
      }
	
   return MarbleOne;
}

function GameConnection::onClientEnterGame(%this)
{
   // Create a new camera object.
   %this.camera = new Camera() {
      dataBlock = Observer;
   };
   MissionCleanup.add( %this.camera );
   %this.camera.scopeToClient(%this);

   // Setup game parameters and create the player
   %this.resetStats();
   %this.spawnPlayer();

   // Anchor the player to the start pad
   %this.player.setMode(Start);

   // Start the game here for single player
   if ($Server::ServerType $= "SinglePlayer")
      startGame();
}

function GameConnection::onClientLeaveGame(%this)
{
   if (isObject(%this.camera))
      %this.camera.delete();
   if (isObject(%this.player))
      %this.player.delete();
}

function GameConnection::resetStats(%this)
{
   // Reset game stats
   %this.bonusTime = 0;
   %this.penaltyTime = 0;
   %this.gemCount = 0;

   // Reset the checkpoint
   //if (isObject(%this.checkPoint))
   //   %this.checkPoint.delete();
   //%this.checkPoint = new ScriptObject() {
   //   pad = $Game::StartPad;
   //   time = 0;
   //   gemCount = 0;
   //   penaltyTime = 0;
   //   bonusTime = 0;
   //   powerUp = 0;
   //};
    $Checkpoint::pad = $Game::StartPad;
	$Checkpoint::time = 0;
	$Checkpoint::gemCount = 0;
	$Checkpoint::penaltyTime = 0;
	$Checkpoint::bonusTime = 0;
	$Checkpoint::powerUp = 0;
}


//-----------------------------------------------------------------------------

function GameConnection::onEnterPad(%this)
{
   if (%this.player.getPad() == $Game::EndPad) {

      if ($Game::GemCount && %this.gemCount < $Game::GemCount) {
         %this.play2d(MissingGemsSfx);
         messageClient(%this, 'MsgMissingGems', '\c0You may not finish without all the diamonds!');
      }
      else {
         %this.player.setMode(Victory);
         messageClient(%this, 'MsgRaceOver', '\c0Congratulations! You\'ve finished!');
         setGameState("End");
      }
   }
}

function GameConnection::onLeavePad(%this)
{
   // Don't care if the leave
}


//-----------------------------------------------------------------------------
function jojo()
{
	ASSERT("Out of Bounds","You suck. Haha.","ASSERT(\"Error Handler\",\"Due to the Out of Bounds addon notifying us that you reached another 1,250 times that you went Out of Bounds, Marble Blast Platinum will now co-operate with it and will now close.\\n\\nGoodbye.\",\"quit();\");");
}
function OOBCounter::check()
{
    %oobRandom[0] = "Let's be clear of the blatant truth: You suck!";
	%oobRandom[1] = "Honestly, do you have any control over the marble? It seems to have a life on its own...";
	%oobRandom[2] = "Are you sure you know how to play Marble Blast?";
	%oobRandom[3] = "You are contributing to the increasing water levels in the sea below you way too much!";
	%oobRandom[4] = "Look at the bright side, it's part of the learning experience, but it doesn't change the fact that you still suck.";
	%oobRandom[5] = "If we ever had a 'You Suck' achievement, you'd be having the honour to wear it today.";
	%oobRandom[6] = "200 more times to go Out of Bounds before you see this message again. For your sake, try and do better.";
	%oobRandom[7] = "\"I didn't play on the computer! It...it was.. my auntie!\" Yeah, right. Admit it, you suck.";
	%oobRandom[8] = "Are you having fun going Out of Bounds all the time? It seriously looks like it.";
	%oobRandom[9] = "Don't you just hate all these messages that make a mockery of your suckiness? It's a joke of course, but it's a nice easter egg.\nIf you don't want to see them anymore, then stop going Out of Bounds so many times!";
	%oobRandom[10] = "My grandma is better than you.";
	
	%oobSpecial[0] = "You went Out of Bounds for 1,250 times. This program will now close, sit in the corner and cry about how bad you are and hope that when you open it again you won't repeat it. False hopes are still hopes.";
	%oobSpecial[1] = "You went Out of Bounds for 2,500 times. If you aren't tired of going Out of Bounds all the time, we sure did. Stop it already!";
	%oobSpecial[2] = "Another 1,250 marbles had fallen to the great sea below, and you've reached the 3,750 Out of Bounds mark. You definitely suck. Ah yes, greenpeace would like to see you in court for your \"contribution\" to rising sea levels.";
	%oobSpecial[3] = "If I had a nickel for every marble that fell Out of Bounds I'd be rich right now and all thanks to you. However, I'm not going to give you any money. Instead, I'll stick my tongue out at you and then laugh at you. Ah yes, congratulations on hitting the 5,000 Out of Bounds mark.";
	%oobSpecial[4] = "I have an awesome gut feeling that you are going 7,500 times Out of Bounds on purpose if only to see these messages and to hear about how bad you are.\nWell then, I won't keep it away from you.\nYou suck!";
	%oobSpecial[5] = "8,750 times Out of Bounds. For reaching this landmark, I'm giving you a nice Australian Slang sentence to answer the question: Will you ever stop sucking in this game and go Out of Bounds? Answer:\nTill it rains in Marble Bar\n\n\nIn your language it means:\nNever.";
	%oobSpecial[6] = "That's it! I'm having enough of you. 10,000 times Out of Bounds.\nYou are the worst Marble Blast player to ever lived on this planet.\nGo away, stop playing and honestly if it weren't a children's game I would TOTALLY curse you until tomorrow.\nI'm not going to continue and think of new messages just for your enjoyment because I'm tired of you and your crappy Marble Blasting skills.\nFrom now on, you get my old messages only, but remember this:\n\n\nYOU SUCK!";

    switch($PREF::OOBCOUNT) {
		case 1250:
			ASSERT("Out of Bounds",%oobSpecial[0],"oobcrash();");
			return 1;
		case 2500:
			ASSERT("Out of Bounds",%oobSpecial[1],"oobcrash();");
			return 1;
		case 3750:
			ASSERT("Out of Bounds",%oobSpecial[2],"oobcrash();");
			return 1;
		case 5000:
			ASSERT("Out of Bounds",%oobSpecial[3],"oobcrash();");
			return 1;
		case 7500:
			ASSERT("Out of Bounds",%oobSpecial[4],"oobcrash();");
			return 1;
		case 8750:
			ASSERT("Out of Bounds",%oobSpecial[5],"oobcrash();");
			return 1;
		case 10000:
			ASSERT("Out of Bounds",%oobSpecial[6],"oobcrash();");
			return 1;
	}
 	
	if($PREF::OOBCOUNT != 0 && $PREF::OOBCOUNT % 200 == 0)
        ASSERT("Out of Bounds" SPC $PREF::OOBCOUNT SPC "times",%oobRandom[getRandom(0,10)]);
		
}

function oobcrash()
{
	schedule(500, 0, "oobassert");
}

function oobassert() {
	ASSERT("Error Handler","Due to the Out of Bounds addon notifying us that you reached another 1,250 times that you went Out of Bounds, Marble Blast Platinum will now co-operate with it and will now close.\n\nGoodbye.","quit();");
}

function GameConnection::onOutOfBounds(%this)
{
   if ($Game::State $= "End")
      return;

   %this.player.setPowerUp(0);
   // Reset the player back to the last checkpoint
   PlayGui.setMessage("outOfBounds",2000);
   %this.play2d(OutOfBoundsVoiceSfx);
   %this.player.setOOB(true);

    // SPY47 - HOORAY FOR THE OOB COUNTER !!!
    $PREF::OOBCOUNT++;
	if( OOBCounter::check() )
		echo("You got owned.");

   if(!isEventPending(%this.respawnSchedule))
   {
      // If checkpointed, don't do READY, SET, GO.
      if($Game::Checkpointed)
         %this.respawnSchedule = %this.schedule(2500, respawnonCheckpoint);
      else
      %this.respawnSchedule = %this.schedule(2500, respawnPlayer);
   }
}

function Marble::onOOBClick(%this)
{
   if($Game::State $= "Play")
   {
      // If checkpointed, don't do READY, SET, GO.
      if($Game::Checkpointed)
         ClientGroup.getObject(0).respawnOnCheckpoint();
      else
         ClientGroup.getObject(0).respawnPlayer();
   }
}

function GameConnection::onDestroyed(%this)
{
   if ($Game::State $= "End")
      return;

   // Reset the player back to the last checkpoint
   PlayGui.setMessage("destroyed",2000);
   %client.play2d(DestroyedVoiceSfx);
   %this.player.setOOB(true);
   if(!isEventPending(%this.respawnSchedule))
      %this.respawnSchedule = %this.schedule(2500, respawnPlayer);
}

function GameConnection::onFoundGem(%this,%amount)
{
   %this.gemCount += %amount;
   %remaining = $Game::gemCount - %this.gemCount;
   if (%remaining <= 0) {
      messageClient(%this, 'MsgHaveAllGems', '\c0You have all the diamonds, head for the finish!');
      %this.play2d(GotAllGemsSfx);
      %this.gemCount = $Game::GemCount;
   }
   else
   {
      if(%remaining == 1)
         %msg = '\c0You picked up a diamond!  Only one diamond to go!';
      else
         %msg = '\c0You picked up a diamond!  %1 diamonds to go!';

      messageClient(%this, 'MsgItemPickup', %msg, %remaining);
      %this.play2d(GotGemSfx);
   }

   PlayGui.setGemCount(%this.gemCount);
}


//-----------------------------------------------------------------------------

function GameConnection::spawnPlayer(%this)
{
   // Combination create player and drop him somewhere
   %spawnPoint = %this.getCheckpointPos(0,"0 0 3", "0 0 0");
   %this.createPlayer(%spawnPoint);
   serverPlay2d(spawnSfx);
}

function restartLevel()
{
   LocalClientConnection.respawnPlayer();
}

function GameConnection::respawnPlayer(%this)
{
   // Reset the player back to the last checkpoint
   cancel(%this.respawnSchedule);
   onMissionReset();
   %this.player.setOOB(false);
   %this.player.setMode(Start);
   %this.player.setPosition(%this.getCheckpointPos(0,"0 0 3", "0 0 0"), 0.45);
   %this.player.setPowerUp($Checkpoint::powerUp,true);

   %this.gemCount = $Checkpoint::gemCount;
   %this.penaltyTime = $Checkpoint::penaltyTime;
   %this.bonusTime = $Checkpoint::bonusTime;

   PlayGUI.setTime($Checkpoint::time);
   PlayGui.setGemCount(%this.gemCount);
   serverPlay2d(spawnSfx);
}

function setPowerUpOnCheckpoint(%player)
{
   %player.player.setPowerUp(%player.checkPoint.powerUp,true);
}

// SPY47 FUNCTION...
function GameConnection::respawnOnCheckpoint(%this)
{
   //echo("DEBUG: Gravitydir:" SPC %this.checkPoint.gravityDir);

   // Reset the player back to the last checkpoint
   cancel(%this.respawnSchedule);

   //setGravityDir("1 0 0 0 -1 0 0 0 -1",true);
   endFireWorks();

   %this.player.setOOB(false);
   %this.player.setMode(End);

   //echo("DEBUG: Reading direction: " @ %this.checkPoint.direction);
   %this.player.setPosition(%this.getCheckpointPos(0,$Checkpoint::add,$Checkpoint::sub), 0.45);

   schedule($powerupDelay, 0, "setPowerUpOnCheckpoint", %this);
   //%this.player.setPowerUp(%this.checkPoint.powerUp,true);

   %this.gemCount = $Checkpoint::gemCount;
   %this.penaltyTime = $Checkpoint::penaltyTime;
   %this.bonusTime = $Checkpoint::bonusTime;
   respawnGems(MissionGroup); // Spy47 : Respawn gems.
   //echo("DEBUG: Respawn gems");
   //echo("DEBUG: Gravitydir:" SPC %this.checkPoint.gravityDir);

   setGravityDir($Checkpoint::gravityDir, true); // Spy47 : Set Gravitydir.
   $Game::GravityDir = $Checkpoint::gravityDir;
   //echo("LOADING GRAVITY: " @ %this.checkPoint.gravityDir);

   // PlayGUI.setTime(%this.checkPoint.time); // Spy47 : keep timer running.
   PlayGui.setGemCount(%this.gemCount);
   serverPlay2d(spawnSfx);
   setGameState("Play"); // Spy47 : Remove onscreen message
}
//-----------------------------------------------------------------------------

function GameConnection::createPlayer(%this, %spawnPoint)
{
   if (%this.player > 0)  {
      // The client should not have a player currently
      // assigned.  Assigning a new one could result in
      // a player ghost.
      error( "Attempting to create an angus ghost!" );
   }
   
    if($playingDemo)
	{
		if(%demo.marble >= 1 || %demo.marble <= $Marbles::NumMarbles)
		{
			%player = new Marble() {
 		  	   // Spy47 check which marble we want
 		  	   dataBlock = %this.getMarbleChoice($demoMarble);
 		  	   client = %this;
		  	 };
		}
		else
		{
			%player = new Marble() {
 		  	   // Spy47 check which marble we want
 		  	   dataBlock = %this.getMarbleChoice(1);
 		  	   client = %this;
		  	 };
		}
	}
	else
	{
		if($pref::CustomMarble)
		{
 			  %player = new Marble() {
 			     // Spy47 check which marble we want
 			     dataBlock = CustomMarble;
 			     client = %this;
			   };
		} else {
 			  %player = new Marble() {
 			     // Spy47 check which marble we want
 			     dataBlock = %this.getMarbleChoice($pref::marbleChoice);
 			     client = %this;
			   };
		}
	}
   MissionCleanup.add(%player);

   // Player setup...
   %player.setPosition(%spawnPoint, 0.45);
   %player.setEnergyLevel(60);
   %player.setShapeName(%this.name);
   %player.client.status = 1;

   // Update the camera to start with the player
   %this.camera.setTransform(%player.getEyeTransform());

   // Give the client control of the player
   %this.player = %player;
   %this.setControlObject(%player);
}


//-----------------------------------------------------------------------------

function GameConnection::setCheckpointTrigger(%object,%player)
{
   // Store the last checkpoint which will be used to restore
   // the player when he goes out of bounds.

   // SPY47 DO WEIRD STUFF YOU WON'T UNDERSTAND NO MATTER YOU TRY.

   %respawnPoint = %object.respawnPoint;
   

   if(%respawnPoint $= "")
   {
	   ASSERT("Error Handler", "You didn't specify the DTS name for respawning!\nPlease refer to the readme.");
	   return;
   }
   %dts = GameConnection::getCheckpointDTS(MissionGroup, %respawnPoint);

   // Nice and beautiful errors.
   if(%dts == -1)
   {
      ASSERT("Error Handler", "Can't find the respawn point!\nMake sure you made a DTS for respawning over.\nPlease refer to the readme.");
	  return;
   }
   
   
   %objectType = %dts.getClassName();
  //echo("DEBUG: Classname " @ %objectType);
   %objectShape = %dts.shapeName;
  //echo("DEBUG: Objectshape " @ %objectShape);

  if(%objectType !$= "TSStatic")
   {
	   ASSERT("Error Handler", "You must specify a TSStatic shape for respawning on!");
	   return;
   }
   if(%objectShape !$= "marble/data/shapes/buttons/checkpoint.dts")
   {
	   //ASSERT("You got owned", "HAHA YOU GOTTA USE CHECKPOINT.DTS!!!!\nWANTED TO USE ANOTHER DTS!! DREAM ON\nHOW LOND DO U THINK IT TOOK TO MAKE CHECKPOINT.DTS HUH??\nAFTER SO MANY HOURS MAKING THE FUCKING DTS SO YOU COME AND USE ANOTHER ONE???\nFUCK OFF, GO TO THE LE AND SELECT CHECKPOINT.DTS!!!");
	   ASSERT("Error Handler", "You have to use marble/data/shapes/buttons/checkpoint.dts as the checkpoint DTS!");
	   return;
   }

    //echo("DEBUG: Current gravity dir:" SPC $Game::GravityDir);
   //echo("DEBUG: Saving direction: " @ %object.direction);
   //if (%object != %player.checkPoint.pad) {
   //   %player.checkPoint.delete();
   //   %player.checkPoint = new ScriptObject() {
   //      pad = %object;
   //      time = %gravity;
   //      gemCount = %player.gemCount;
   //      penaltyTime = %player.penaltyTime;
   //      bonusTime = %player.bonusTime;
   //      powerUp = %player.player.getPowerUp();
   //      add = %object.add;
   //      sub = %object.sub;
   //      gravityDir = $Game::GravityDir;
   //   };
    $Checkpoint::pad = %object;
	$Checkpoint::time = %gravity;
	$Checkpoint::gemCount = %player.gemCount;
	$Checkpoint::penaltyTime = %player.penaltyTime;
	$Checkpoint::bonusTime = %player.bonusTime;
	$Checkpoint::powerUp = %player.player.getPowerUp();
	$Checkpoint::add = %object.add;
	$Checkpoint::sub = %object.sub;
	$Checkpoint::gravityDir = $Game::GravityDir;
	
	
      messageClient(%player, 'MsgCheckpoint', "\c0Checkpoint reached!");
      serverPlay2d(CheckpointSfx);
      //echo("gem count" @ %player.gemCount);
      $Game::Checkpointed = 1; // Spy47 : We've hit a checkpoint once.

      $Game::CurCheckpoint = %object;
	  $Game::CurCheckpointNum++;
   }




function GameConnection::setCheckpointButton(%object,%player)
{
   // This is for checkpoint button.

   // SPY47 DO WEIRD STUFF YOU WON'T UNDERSTAND NO MATTER YOU TRY.
   
   //echo("DEBUG: Current gravity dir:" SPC $Game::GravityDir);
   //echo("DEBUG: Saving direction: " @ %object.direction);
   //if (%object != %player.checkPoint.pad) {
   //   %player.checkPoint.delete();
   //   %player.checkPoint = new ScriptObject() {
   //      pad = %object;
   //      time = %gravity;
   //      gemCount = %player.gemCount;
   //      penaltyTime = %player.penaltyTime;
   //      bonusTime = %player.bonusTime;
   //      powerUp = %player.player.getPowerUp();
   //      add = %object.add;
   //      sub = %object.sub;
   //      gravityDir = $Game::GravityDir;
   //   };
    $Checkpoint::pad = %object;
	$Checkpoint::time = %gravity;
	$Checkpoint::gemCount = %player.gemCount;
	$Checkpoint::penaltyTime = %player.penaltyTime;
	$Checkpoint::bonusTime = %player.bonusTime;
	$Checkpoint::powerUp = %player.player.getPowerUp();
	$Checkpoint::add = %object.add;
	$Checkpoint::sub = %object.sub;
	$Checkpoint::gravityDir = $Game::GravityDir;
	
      messageClient(%player, 'MsgCheckpoint', "\c0Checkpoint reached!");
      serverPlay2d(CheckpointSfx);
      //echo("gem count" @ %player.gemCount);
      $Game::Checkpointed = 1; // Spy47 : We've hit a checkpoint once.

      $Game::CurCheckpoint = %object;
	  $Game::CurCheckpointNum++;
   }






// CHECKPOINT BUTTON SPY47
function CheckpointTrigger::onEnterTrigger(%this,%obj,%col)
{
   //echo("got into trigger");
   //%obj.setCheckpoint($Game::Checkpoint[%this.checkpointNum]);

   %player = %col.client;

   //if(!%obj.checkpointNum)
   //   echo("ERROR!!! No checkpointNum specified.");
   if( %obj != $Game::CurCheckpoint )
   {
      GameConnection::setCheckpointTrigger(%obj,%col.client);
   }
}

function checkPoint::onCollision(%this,%obj,%col,%vec, %vecLen, %material)
{
   //echo("got into trigger");
   //if(!%obj.checkpointNum)
   //   echo("ERROR!!! No checkpointNum specified.");

   %player = %col.client;
   if( %obj != $Game::CurCheckpoint )
   {
      GameConnection::setCheckpointButton(%obj,%col.client);
   }
}

function GameConnection::getCheckpointPos(%this,%num,%add,%sub)
{
   // Return the point a little above the object's center
   if (!isObject($Checkpoint::pad))
      return "0 0 300 1 0 0 0";

   //echo("DEBUG: direction is " @ %add @ %sub);
   // SPY47 MODIFICATION

   // If no ADD and SUB was specified... respawn normally
   if(%add $= "" && %sub $= "")
      return vectorAdd($Checkpoint::pad.getTransform(),"0 0 3") SPC getWords($Checkpoint::pad.getTransform(), 3);
   

   %vector = vectorAdd($Checkpoint::pad.getTransform(),%add);
   %vector = vectorSub(%vector,%sub);
   //return vectorAdd(%this.checkPoint.pad.getTransform(),"0 0 3") SPC getWords(%this.checkPoint.pad.getTransform(), 3);
   return %vector SPC getWords($Checkpoint::pad.getTransform(), 3);

}

function GameConnection::getCheckpointDTS(%group, %respawnPoint)
{
   for (%i = 0; %i < %group.getCount(); %i++)
   {
      %object = %group.getObject(%i);
      %type = %object.getClassName();
	  %name = %object.getName();
	  //echo("This object is called " @ %name @ ", but destination is " @ %destination);
         if (%name $= %respawnPoint)
            return %object;
   }
   return -1;
}



//-----------------------------------------------------------------------------
// Support functions
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

function countGems(%group)
{
   // Count up all gems out there are in the world
   %gems = 0;
   for (%i = 0; %i < %group.getCount(); %i++)
   {
      %object = %group.getObject(%i);
      %type = %object.getClassName();
      if (%type $= "SimGroup")
         %gems += countGems(%object);
      else
         if (%type $= "Item" &&
               %object.getDatablock().classname $= "Gem")
            %gems++;
   }
   return %gems;
}

// Spy47 : Respawn gems not in checkpoint.
function respawnGems(%group)
{
   //echo("DEBUG: Starting respawnGems.");
   // Count up all gems out there are in the world
   for (%i = 0; %i < %group.getCount(); %i++)
   {
      %object = %group.getObject(%i);
      %type = %object.getClassName();
      if (%type $= "SimGroup")
         respawnGems(%object);
      else
         if (%type $= "Item" && %object.getDatablock().classname $= "Gem")
         {
             //echo("DEBUG: This is gem. pickUpCheckpoint:" SPC %object.pickUpCheckpoint SPC "CurCheckpoint:" SPC $Game::CurCheckpointNum);
             if(%object.pickUpCheckpoint >= $Game::CurCheckpointNum)
             {
                 %object.onMissionReset(); // Spy47 : respawn gem.
                 %object.pickUpCheckpoint = 0;
                 //echo("DEBUG: Gem respawned.");
             }
         }
   }
}
